/*
    UHU-menu -- output plugin for OpenBox (3.2)
    Copyright (C) 2004  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhu-menu.h"

static void print_name (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "-");
		return;
	}
	while (*s != '\0') {
		switch (*s) {
		  case '\"':
			fprintf(f, "&quot;");
			break;
		  case '<':
			fprintf(f, "&lt;");
			break;
		  case '>':
			fprintf(f, "&gt;");
			break;
		  case '&':
			fprintf(f, "&amp;");
			break;
		  default:
			fputc(*s, f);
		}
		s++;
	}
}

static void print_exec (FILE *f, const char *s)
{
	if (s == NULL) {
		fprintf(f, "false");
		return;
	}
	while (*s != '\0') {
		if (*s == '%') {
			s++;
			if (*s == '%' || *s == '\0') fputc('%', f);
			if (*s != '\0') s++;
			continue;
		}
		switch (*s) {
		  case '<':
			fprintf(f, "&lt;");
			break;
		  case '>':
			fprintf(f, "&gt;");
			break;
		  case '&':
			fprintf(f, "&amp;");
			break;
		  default:
			fputc(*s, f);
		}
		s++;
	}
}

static void dump (FILE *f, const struct item *item, const char *lang)
{
	int i;
	int major = 0;
	for (i = 0; i < item->subitems_nr; i++) {
		if (item->subitems[i].issubmenu) {
			dump(f, &item->subitems[i], lang);
		}
	}
	fprintf(f, "<menu id=\"UHU%s\" label=\"", item->file_name);
	print_name(f, get_value(item, NULL, "Name", lang));
	fprintf(f, "\">\n");
	for (i = 0; i < item->subitems_nr; i++) {
		if (i > 0 && item->subitems[i].major != major) {
			fprintf(f, "  <separator/>\n");
		}
		if (!item->subitems[i].issubmenu) {
			fprintf(f, "  <item label=\"");
			print_name(f, get_value(&item->subitems[i], NULL, "Name", lang));
			fprintf(f, "\">\n    <action name=\"Execute\"><execute>%s",
			  item->subitems[i].terminal ? "terminal " : "");
			print_exec(f, get_value(&item->subitems[i], NULL, "Exec", lang));
			fprintf(f, "</execute></action>\n  </item>\n");
		} else {
			fprintf(f, "  <menu id=\"UHU%s\"/>\n", item->subitems[i].file_name);
		}
		major = item->subitems[i].major;
	}
	fprintf(f, "</menu>\n\n");
}

int write_menu_lang (const struct item *item, const char *path,
  const struct language *language)
{
	char *lang;
	char file[PATH_MAX];
	FILE *f;

	lang = language->name;
	sprintf(file, "%s/menu%s%s.xml", path, lang[0] ? "." : "", lang);
	if ((f = fopen(file, "w")) == NULL) {
		message_e(3, "fopen %s", file);
		return -1;
	}
	fprintf(f, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n");
	fprintf(f, "<openbox_menu xmlns=\"http://openbox.org/\"\n");
	fprintf(f, "        xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
	fprintf(f, "        xsi:schemaLocation=\"http://openbox.org/\n");
	fprintf(f, "                file:///usr/share/openbox/menu.xsd\">\n\n");
	dump(f, item, lang);
	fprintf(f, "</openbox_menu>\n");
	fclose(f);
	return 0;
}

